<?php

namespace Drupal\commerce_payment_paw\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItemInterface;
use Drupal\commerce_order\OrderStorage;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "paw_offsite_redirect",
 *   label = "PayAnyWay (Off-site redirect)",
 *   display_label = "PayAnyWay",
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_payment_paw\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "dinersclub", "discover", "jcb", "maestro", "mastercard", "visa",
 *   },
 * )
 */
class OffsiteRedirect extends OffsitePaymentGatewayBase
{

	/**
	 * {@inheritdoc}
	 */
	public function defaultConfiguration()
	{
		return [
			'mnt_id' => '',
			'mnt_secret' => '',
			'mnt_demo' => false,
			'mnt_test_mode' => false,
			'mnt_kassa' => false
		] + parent::defaultConfiguration();
	}

	/**
	 * {@inheritdoc}
	 */
	public function buildConfigurationForm(array $form, FormStateInterface $form_state)
	{
		$form = parent::buildConfigurationForm($form, $form_state);

		$form['mnt_id'] = [
			'#type' => 'textfield',
			'#title' => $this->t('Account number'),
			'#default_value' => $this->configuration['mnt_id']
		];

		$form['mnt_secret'] = [
			'#type' => 'textfield',
			'#title' => $this->t('Secret code'),
			'#default_value' => $this->configuration['mnt_secret']
		];

		$form['mnt_demo'] = [
			'#type' => 'checkbox',
			'#title' => $this->t('Use demo-account (demo.moneta.ru)'),
			'#default_value' => $this->configuration['mnt_demo']
		];

		$form['mnt_test_mode'] = [
			'#type' => 'checkbox',
			'#title' => $this->t('Test mode'),
			'#default_value' => $this->configuration['mnt_test_mode']
		];

		$form['mnt_kassa'] = [
			'#type' => 'checkbox',
			'#title' => $this->t('Use kassa.payanyway.ru'),
			'#default_value' => $this->configuration['mnt_kassa']
		];

		return $form;
	}

	/**
	 * {@inheritdoc}
	 */
	public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
	{
		parent::submitConfigurationForm($form, $form_state);
		if (!$form_state->getErrors())
		{
			$values = $form_state->getValue($form['#parents']);
			$this->configuration['mnt_id'] = $values['mnt_id'];
			$this->configuration['mnt_secret'] = $values['mnt_secret'];
			$this->configuration['mnt_demo'] = $values['mnt_demo'];
			$this->configuration['mnt_test_mode'] = $values['mnt_test_mode'];
			$this->configuration['mnt_kassa'] = $values['mnt_kassa'];
		}
	}


	const paw_kassa_VAT0     = '1104';  // НДС 0%
	const paw_kassa_VAT10    = '1103';  // НДС 10%
	const paw_kassa_VAT18    = '1102';  // НДС 18%
	const paw_kassa_VATNOVAT = '1105';  // НДС не облагается
	const paw_kassa_VATWR10  = '1107';  // НДС с рассч. ставкой 10%
	const paw_kassa_VATWR18  = '1106';  // НДС с рассч. ставкой 18%

	private static function monetaPayURLResponse($mnt_id, $mnt_transaction_id, $mnt_data_integrity_code, $success = false,
												 $repeatRequest = false, $echo = true, $die = true, $kassa_inventory = null, $kassa_customer = null, $kassa_delivery = null)
	{
		if ($success === true)
			$resultCode = '200';
		elseif ($repeatRequest === true)
			$resultCode = '402';
		else
			$resultCode = '500';
		$mnt_signature = md5($resultCode.$mnt_id.$mnt_transaction_id.$mnt_data_integrity_code);
		$response = '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
		$response .= "<MNT_RESPONSE>\n";
		$response .= "<MNT_ID>{$mnt_id}</MNT_ID>\n";
		$response .= "<MNT_TRANSACTION_ID>{$mnt_transaction_id}</MNT_TRANSACTION_ID>\n";
		$response .= "<MNT_RESULT_CODE>{$resultCode}</MNT_RESULT_CODE>\n";
		$response .= "<MNT_SIGNATURE>{$mnt_signature}</MNT_SIGNATURE>\n";
		if (!empty($kassa_inventory) || !empty($kassa_customer) || !empty($kassa_delivery))
		{
			$response .= "<MNT_ATTRIBUTES>\n";
			foreach (array('INVENTORY' => $kassa_inventory, 'CUSTOMER' => $kassa_customer, 'DELIVERY' => $kassa_delivery) as $k => $v)
				if (!empty($v))
					$response .= "<ATTRIBUTE><KEY>{$k}</KEY><VALUE>{$v}</VALUE></ATTRIBUTE>\n";
			$response .= "</MNT_ATTRIBUTES>\n";
		}
		$response .= "</MNT_RESPONSE>\n";
		if ($echo === true)
		{
			header("Content-type: application/xml");
			echo $response;
		}
		else
			return $response;
		if ($die === true)
			die;
		return '';
	}


	/**
	 * PayURL handler
	 * URL: http://site.domain/payment/notify/payanyway
	 *
	 * {@inheritdoc}
	 *
	 * @param Request $request
	 * @return null|void
	 */
	public function onNotify(Request $request)
	{
		try
		{
			// Получаем параметры
			$mnt_id = $request->get('MNT_ID');
			$mnt_transaction_id = $request->get('MNT_TRANSACTION_ID');
			$mnt_operation_id = $request->get('MNT_OPERATION_ID');
			$mnt_amount = $request->get('MNT_AMOUNT');
			$mnt_currency_code = $request->get('MNT_CURRENCY_CODE');
			$mnt_subscriber_id = $request->get('MNT_SUBSCRIBER_ID');
			$mnt_test_mode = $request->get('MNT_TEST_MODE');
			$mnt_signature = $request->get('MNT_SIGNATURE');
			$mnt_secret = $this->configuration['mnt_secret'];
			$mnt_kassa = $this->configuration['mnt_kassa'];
			// Проверяем, все ли параметры были переданы (если $request->get отдал null, значит параметр не был передан)
			if (is_null($mnt_id) ||
				is_null($mnt_transaction_id) ||
				is_null($mnt_operation_id) ||
				is_null($mnt_amount) ||
				is_null($mnt_currency_code) ||
				is_null($mnt_subscriber_id) ||
				is_null($mnt_test_mode) ||
				is_null($mnt_signature))
			{
				throw new Exception('Required input parameter(s) not found');
			}
			// Проверяем, совпадает ли MNT_ID со значением из конфигурации
			if ($this->configuration["mnt_id"] !== $mnt_id)
			{
				throw new Exception('MNT_ID mismatch');
			}
			// Загружаем ORDER с соответствующим ID
			/** @var $order_storage OrderStorage */
			$order_storage = $this->entityTypeManager->getStorage('commerce_order');
			/** @var $order OrderInterface */
			$order = $order_storage->load($mnt_transaction_id);
			if (is_null($order))
			{
				throw new Exception('Order not found');
			}
			// Сравниваем суммы (принятую и записанную в ORDER'е)
			$amount = number_format($order->getTotalPrice()->getNumber(), 2, '.', '');
			if ($mnt_amount !== $amount)
			{
				throw new Exception('Payment amount mismatch');
			}
			// Проверяем подпись
			$signature = md5($mnt_id.$mnt_transaction_id.$mnt_operation_id.$amount.$mnt_currency_code.$mnt_subscriber_id.$mnt_test_mode.$mnt_secret);
			if ($signature !== $mnt_signature)
			{
				throw new Exception('Payment signature mismatch');
			}
			// Меняем статус ORDER'а и сохраняем его
			$order->set('state','completed');
			$order->save();
			// Создаём платёж и сохраняем его
			$payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
			$payment = $payment_storage->create([
				'state' => 'authorization',
				'amount' => $order->getTotalPrice(),
				'payment_gateway' => $this->entityId,
				'order_id' => $order->id(),
				'test' => $this->getMode() == 'test',
				'remote_id' => $mnt_operation_id,
				'remote_state' => 'PAYED',
				'authorized' => REQUEST_TIME,
			]);
			$payment->save();

			// KASSA ADDON
			if ($mnt_kassa == true)
			{
				$inventoryPositions = [];

				$clientEmail = $payment->getOrder()->getEmail();

				/** @var OrderItemInterface[] $items */
				$items = $payment->getOrder()->getItems();
				/** @var OrderItemInterface $item */
				foreach($items as $item)
				{
					$taxMode = self::paw_kassa_VATNOVAT;
					$inventoryPositions[] = [
						'name' => trim(preg_replace("/&?[a-z0-9]+;/i", "", htmlspecialchars($item->getTitle()))),
						'price' => $item->getUnitPrice()->getNumber(),
						'quantity' => $item->getQuantity(),
						'vatTag' => $taxMode
					];
				}

				$kassa_inventory = json_encode($inventoryPositions);

				self::monetaPayURLResponse($_REQUEST['MNT_ID'], $_REQUEST['MNT_TRANSACTION_ID'], $mnt_secret, true,
					false, true, true, $kassa_inventory, $clientEmail);											// 200
			}
			// END OF KASSA ADDON

			// Если не произошло исключений, выдаём "SUCCESS"
			echo 'SUCCESS';
		}
		catch (Exception $e)
		{
			watchdog_exception('PayURL handler', $e);
			echo 'FAIL';
		}
	}

	/**
	 * Page for MNT_SUCCESS_URL
	 *
	 * {@inheritdoc}
	 *
	 * @param OrderInterface $order
	 * @param Request $request
	 */
	public function onReturn(OrderInterface $order, Request $request)
	{
		drupal_set_message('Payment was processed');
	}
}
